
vendor_map = {}
vendor_map['9'] = 'cisco'
vendor_map['2011'] = 'huawei'
vendor_map['2352'] = 'ericsson'
vendor_map['2636'] = 'juniper'
vendor_map['6527'] = 'alcatel'
vendor_map['8072'] = 'net-snmp'
vendor_map['30065'] = 'arista'
vendor_map['43191'] = 'metamako'
